#
#
#   API
#
#

import os
import struct

from .utils import human_size


CHUNK_SIZE = 65536  # 64K: 64 * 1024
CHUNK_BYTE_FORMAT = "<q"  # long long little-endian


def to_uint64(value):
    return value & 0xFFFFFFFFFFFFFFFF


def oshash(file_path: str):
    byte_size = struct.calcsize(CHUNK_BYTE_FORMAT)

    file_size = os.path.getsize(file_path)

    min_file_size = CHUNK_SIZE * 2  # head and tail

    if file_size < min_file_size:
        raise ValueError("File size must be at least {}".format(human_size(min_file_size)))

    with open(file_path, "rb") as f:
        file_hash = file_size

        num_chunk_iterations = CHUNK_SIZE // byte_size

        for _ in range(num_chunk_iterations):
            file_bytes = f.read(byte_size)
            [unpacked] = struct.unpack(CHUNK_BYTE_FORMAT, file_bytes)
            file_hash = to_uint64(file_hash + unpacked)  # make sure variable is int64

        f.seek(-CHUNK_SIZE, os.SEEK_END)  # go to tail

        for _ in range(num_chunk_iterations):
            file_bytes = f.read(byte_size)
            [unpacked] = struct.unpack(CHUNK_BYTE_FORMAT, file_bytes)
            file_hash = to_uint64(file_hash + unpacked)

        return format(file_hash, "016x")  # convert to hex and pad with zeros if needed



import os
import setuptools

here = os.path.abspath(os.path.dirname(__file__))

with open("README.md", "r") as fp:
    long_description = fp.read()

about = {}
with open(os.path.join(here, "oshash", "__version__.py"), "r") as f:
    exec(f.read(), about)

setuptools.setup(
    name="oshash",
    version=about["__version__"],
    author=about["__author__"],
    description=about["__description__"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=about["__url__"],
    license=about["__license__"],
    packages=setuptools.find_packages(),
    entry_points={"console_scripts": ["oshash = oshash.command:main"]},
    python_requires=">=3.5",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers"
    ]
)



#
#
#
#
#

import os
import sys
import argparse

from .api import oshash


def main():
    parser = argparse.ArgumentParser(description="OpenSubtitles Hash tool")

    parser.add_argument("file_path")

    args = parser.parse_args()

    file_path = os.path.expanduser(args.file_path)

    try:
        file_hash = oshash(file_path)
    except (IOError, ValueError) as e:
        sys.exit(e)

    print("OSHash ({}) = {}".format(file_path, file_hash))


if __name__ == '__main__':
    main()



#
#
#   Utils
#
#


def human_size(size_bytes):
    """
    format a size in bytes into a 'human' file size, e.g. bytes, KB, MB, GB, TB, PB
    Note that bytes/KB will be reported in whole numbers but MB and above will have greater precision
    e.g. 1 byte, 43 bytes, 443 KB, 4.3 MB, 4.43 GB, etc
    """
    if size_bytes == 1:
        # because I really hate unnecessary plurals
        return "1 byte"

    suffixes_table = [('bytes', 0), ('KB', 0), ('MB', 1), ('GB', 2), ('TB', 2), ('PB', 2)]

    num = float(size_bytes)
    for suffix, precision in suffixes_table:
        if num < 1024.0:
            break
        num /= 1024.0

    if precision == 0:
        formatted_size = "%d" % num
    else:
        formatted_size = str(round(num, ndigits=precision))

    return "%s %s" % (formatted_size, suffix)
